# Automatiser les tâches récurrentes

## Utiliser le langage de ligne de commande Bash

Mon outil préféré pour scripter des tâches que j'ai à effectuer de façon récurrente est Bash. C'est un langage de script qui est principalement utilisé sur les systèmes d'exploitation GNU/Linux et macOS.

Il est aussi possible d'installer [MSYS2](https://www.msys2.org/) sur Windows pour obtenir accès à une multitude de logiciels créés initialement pour GNU/Linux, dont Bash.

Parmi les fonctionnalités les plus fréquemment utilisées du langage de script Bash, on retrouve:

- Appeler des logiciels
- La manipulation de fichiers
- Les tubes (*pipes* en anglais),
- Les boucles

Allons plus en détail sur chacune d'entre elles avec des exemples!

### L'appel de logiciels

Le langage de script Bash permet d'interagir avec tous les logiciels présents sur votre environnement de travail. Il permet surtout de créer des séquences de tâches exécutées les unes après les autres en suivant des conditions spécifiques. Bref, c'est un moyen de faire travailler notre ordinateur à notre place!

Un exemple d'appel de logiciels que j'utilise souvent, c'est un court programme qui est écrit dans le langage Ruby qui permet de convertir récursivement en minuscules des noms des répertoires et des fichiers. Il est très utile pour s'assurer que les fichiers musicaux que je souhaite mettre sur mon téléphone Android soient bien reconnus.

Il peut être appelé depuis Bash comme suit:

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
mon_code="Dir['*'].each { |p| File.rename(p, p.downcase) }"
ruby -e ${mon_code}
```

- Le script Bash écrit la chaîne de caractère qui contient le code Ruby dans la variable `mon_code`.
- Ensuite, il envoie le contenu de `mon_code` à l'interpréteur du langage Ruby

### La manipulation de fichiers

Au lieu d'utiliser des fichiers complexes tels que ceux produits par les suites bureautiques tel que Microsoft Office, les environnements en ligne de commande préfèrent les fichiers simples au format texte. Ils excellent dans la manipulation de ceux-ci.

Souvent, les logiciels conçus pour la ligne de commande ont une interface très simple: ils reçoivent du texte, souvent par l'intermédiaire d'un fichier, et vont retourner du texte, souvent aussi dans un fichier. Dans les autres cas, le texte va voyager d'un programme à l'autre via les tubes.

Un exemple fréquent de manipulation de fichiers est la concaténation de fichiers pdf. On veut fusionner plusieurs fichiers ensemble en un seul document. Le logiciel pdftk permet de faire cette opération comme suit:

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
pdftk \
page_couverture.pdf \
table_matieres.pdf \ 
chapitre_1.pdf \
chapitre_2.pdf \ 
conclusion.pdf \
cat output mon_ebook.pdf
```

- On lance le logiciel `pdftk`
- Ensuite, on énumère dans l'ordre tous les fichiers à concaténer
- On spécifie l'action. Le mot `cat` est souvent utilisé pour la concaténation.
- On spécifie le fichier de sortie après `output`. Ici, c'est `mon_ebook.pdf`

Petite note: la barre oblique `\` c'est pour couper les lignes pour les rendre plus faciles à lire !

### Les tubes

Les tubes permettent d'utiliser la sortie d'un programme directement dans l'entrée d'un autre programme, sans devoir écrire un fichier. Ils sont représentés par la barre verticale `|`.

Un exemple fréquent que j'utilise est pour effectuer la recherche de fichiers.

Le logiciel le plus courant sur GNU/Linux qui permet de listes tous les fichiers d'un répertoire se nomme `find`. Sa tâche est de listes tous les fichiers qui répondent à certaines conditions, par exemple une extension précise comme `.pdf` pour les fichiers PDF. Si je veux filtrer plus précisément cette liste de fichiers avec d'autres attributs, je vais utilise le logiciel grep qui filtre le texte qu'on lui fournis en entrée selon une expression régulière, qui est une syntaxe fréquemment utilisée dans la recherche de texte.

Par exemple, si je veux trouver tous les livres sur le logiciel Spark qui se trouvent dans ma bibliothèque Nextcloud, la commande Bash correspondante prend la forme suivante:

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
find "nextcloud/Livres" -type f | \
grep -i "voyage|travel"
```

- Le premier argument de find est le répertoire de recherche, ici c'est `nextcloud/Livres/Technical`.
- Le premier paramètre est `type`, avec comme valeur `f`, pour rechercher uniquement les fichiers.
- Ensuite, il y a un tube vers la commande `grep`
- `grep` a un paramètre, `-i` qui spécifie que ce qui suit n'est pas sensible à la casse (an anglais *case insentitive*)
- Enfin, il y a l'expression de recherche `voyage|travel` qui va retourner tous les noms de fichiers qui contiennent soit le mot `voyage`, soit le mot `travel`

### Les boucles

Les boucles sont des mécanismes simples et puissants qui permettent de répéter des tâches identiques ou très similaires pour de multiples entrées. Par exemple, si vous voulez produire un document quasi identique pour chaque jour de la semaine, vous pouvez le faire de cette façon. L'élément de code qui permet de répéter du code un nombre déterminé de fois est nommé `for`. Il existe aussi les boucles `while` et `until` qui sont basées sur une condition. Celles-ci sont utiles lorsque le nombre de répétitions n'est pas connu à l'avance.

### Un exemple pratique

Pour créer ce guide, j'utilise le langage [Markdown](https://www.markdownguide.org/) ainsi que le langage [\LaTeX](https://www.latex-project.org/), deux systèmes de conception de documents. Le premier est davantage utilisé pour le contenu texte et le code informatique, alors que le second est davantage pour la mise en page et les graphiques. Comme je dois souvent convertir les fichier Markdown en \LaTeX, puis en PDF pour avoir un aperçu du document, alors j'utilise Bash et les logiciel `pandoc` et `pdflatex`.

Ce guide comprend 3 sections composées chacunes de 4 chapitres. Pour chaque chapitre, j'ai un fichier Markdown distinct. Je veux convertir chacun d'eux vers le format \LaTeX. Je vais donc faire une boucle pour chacune des sections, puis à l'intérieur de celle-ci, faire un boucle pour chacun des chapitres. Mon code sera donc exécuté 12 fois.

Ensuite, j'utilise le logiciel `pdflatex` pour créer le document PDF que vous êtes actuellement en train de feuilleter.

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
#!/bin/bash

for section in {1..3}
do
    for chapitre in {1..4}
    do
        pandoc \
        -f markdown \
        -t latex \
        --listings \
        --top-level-division=chapter \
        -i chapitre_${section}_${chapitre}.md \
        -o ../chapitres/chapitre_${section}_${chapitre}.tex
    done
done

cd ../ # je change de repertoire, les fichiers pour LaTeX sont un niveau plus haut dans mon dossier de travail !
pdflatex -synctex=1 -interaction=nonstopmode main.tex
cd markdown # je reviens au repertoire d'origine
```

Note: [Pandoc](https://pandoc.org/) est un logiciel vraiment utile pour les créateurs de contenu, il permet de tout faire en Markdown (qui est facile à maîtriser), puis de publier sous une multitude de formats. Je vous conseille fortement d'y jeter un oeil.

## Programmer l'exécution de scripts

Nous avons maintenant créé le script parfait qui exécute les tâches ennuyantes que nous faisions auparavant manuellement. Maintenant, puisque nous planifions partir en vacances, il faut dire à notre système quand exécuter le script et à quelle fréquence. C'est ce que `cron` permet de faire sur GNU/Linux et macOS. Sur Windows, c'est le planificateur de tâches `schtasks.exe`.

Notez qu'il existe aussi des applications graphiques pour chacun de ces systèmes. Mais, vous êtes ici pour voir un peu de code non ?

### cron sur Linux et macOS

Le logiciel cron est inclus par défaut sur la majorité des systèmes macOS et GNU/Linux. Il n'y a donc rien à faire pour l'obtenir. Cependant, il est possible qu'il nécessite des accès additionnels pour bien fonctionner sur macOS.

Les instructions pour cron sont inscrites dans un fichier spécial au format `crontab` dans le répertoire `/etc/cron.d`. Il est aussi possible de mettre le script à exécuter dans un des 4 répertoires suivants

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
/etc/cron.daily
/etc/cron.hourly
/etc/cron.monthly
/etc/cron.weekly
```

Le format des instructions dans un fichier `crontab` est défini comme dans cette illustration:

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
*   *   *   *   *   *    script_a_executer.sh
|   |   |   |   |   |
|   |   |   |   |   +--- nom utilisateur
|   |   |   |   +----- jour de la semaine (0-6) (Dimanche=0)
|   |   |   +------- mois (1-12)
|   |   +--------- jour du mois (1-31)
|   +----------- heure (0-23)
+------------- minutes (0-59)
```

Le comportement par défaut est d'exécuter le script à chaque minute. Ensuite, si on veut réduire la fréquence, on spécifie à quel moment exécuter le script.

Par exemple, pour exécuter un script `cocktail_aleatoire.sh` qui nous envoie une recette de cocktail au hasard à 16h30 chaque vendredi, on configure comme suit.

```{.bash linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
30  16  *  *  5  jevalideca /home/jevalideca/cocktail_aleatoire.sh
```

### Planificateur de tâches sur Windows

Sur Windows, l'ensemble des tâches peuvent être planifiées depuis le Planificateur de tâches. En exécutant une commande PowerShell, il est possible de configurer une tâche similaire à la précédente.

À partir d'une session PowerShell, la séquence de commandes suivante configurera l'exécution du programme `cocktail_aleatoire.bat` à 16:30 chaque vendredi.

```{.powershell linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
$action = New-ScheduledTaskAction -Execute 'C:\Users\jevalideca\scripts\cocktail_aleatoire.bat'
$trigger = New-ScheduledTaskTrigger -Weekly -DaysOfWeek Friday -At 4:30pm
Register-ScheduledTask -Action $action -Trigger $trigger -TaskName "cocktail-aleatoire"
```

Une fois la commande exécutée, nous obtenons cette réponse:

```{.powershell linenos="true" breaklines="true" tabsize="2" fontsize="\small" bgcolor="turquoisefonce!10!white"}
TaskPath                                       TaskName                          State
--------                                       --------                          -----
\                                              cocktail-aleatoire                Ready
```

Nous pouvons valider le résultat dans la fenêtre du Planificateur de tâches.

![Capture d'écran du planificateur de tâches](capture_schtasks_cocktail.png)

## Conclusion

J'ai présenté deux techniques qui permettent de scripter et de configurer l'exécution automatique de tâches.

Il est maintenant possible de faire travailler ton ordinateur durant tes vacances !
